@@ -15,11 +15,14 @@
 #ifndef CEPH_AUTHSERVICEHANDLER_H
 #define CEPH_AUTHSERVICEHANDLER_H
 
-#include "include/types.h"
-#include "Auth.h"
+#include <stddef.h>              // for NULL
+#include <stdint.h>              // for uint64_t
+#include "common/entity_name.h"  // for EntityName
+#include "include/buffer_fwd.h"  // for bufferlist
 
 class CephContext;
 class KeyServer;
+struct AuthCapsInfo;
 
 struct AuthServiceHandler {
 protected:
