@@ -31,19 +31,19 @@ class MExportDirFinish : public Message {
     set_tid(tid);
   }
 private:
-  ~MExportDirFinish() {}
+  ~MExportDirFinish() override {}
 
 public:
-  const char *get_type_name() const { return "ExFin"; }
-  void print(ostream& o) const {
+  const char *get_type_name() const override { return "ExFin"; }
+  void print(ostream& o) const override {
     o << "export_finish(" << dirfrag << (last ? " last" : "") << ")";
   }
   
-  void encode_payload(uint64_t features) {
+  void encode_payload(uint64_t features) override {
     ::encode(dirfrag, payload);
     ::encode(last, payload);
   }
-  void decode_payload() {
+  void decode_payload() override {
     bufferlist::iterator p = payload.begin();
     ::decode(dirfrag, p);
     ::decode(last, p);
