@@ -7,15 +7,15 @@
 #ifndef ROCKSDB_LITE
 #include <vector>
 
-#include "db/filename.h"
 #include "db/log_reader.h"
 #include "db/version_set.h"
+#include "options/db_options.h"
 #include "port/port.h"
 #include "rocksdb/env.h"
 #include "rocksdb/options.h"
 #include "rocksdb/transaction_log.h"
 #include "rocksdb/types.h"
-#include "util/db_options.h"
+#include "util/filename.h"
 
 namespace rocksdb {
 
@@ -92,13 +92,10 @@ class TransactionLogIteratorImpl : public TransactionLogIterator {
     Env* env;
     Logger* info_log;
     virtual void Corruption(size_t bytes, const Status& s) override {
-      Log(InfoLogLevel::ERROR_LEVEL, info_log,
-          "dropping %" ROCKSDB_PRIszt " bytes; %s", bytes,
-          s.ToString().c_str());
-    }
-    virtual void Info(const char* s) {
-      Log(InfoLogLevel::INFO_LEVEL, info_log, "%s", s);
+      ROCKS_LOG_ERROR(info_log, "dropping %" ROCKSDB_PRIszt " bytes; %s", bytes,
+                      s.ToString().c_str());
     }
+    virtual void Info(const char* s) { ROCKS_LOG_INFO(info_log, "%s", s); }
   } reporter_;
 
   SequenceNumber currentBatchSeq_; // sequence number at start of current batch
